{ File    : $URL: file:///D:/SVN/svnDelphi/tags/IMA/IMAMgr/1.10.5502.2/src/IMAMgr.dpr $
  Author  : $Author: Deniz Özmen $
  Changed : $Date: 2011-07-02 21:57:50 +0200 (Sa, 02 Jul 2011) $
  Revision: $Rev: 21 $

  IMA ADPCM framework library.
}
library IMAMgr;

{$IMAGEBASE $40100000}

uses
  Handler  in 'base\Handler.pas',
  Manager  in 'base\Manager.pas',
  Shared   in 'base\Shared.pas',
  BS3      in 'formats\BS3.pas',
  Default  in 'formats\Default.pas',
  Driver   in 'formats\Driver.pas',
  Express  in 'formats\Express.pas',
  Jack     in 'formats\Jack.pas',
  TLJ      in 'formats\TLJ.pas',
  XMen2    in 'formats\XMen2.pas',
  SysUtils;

{$R *.res}

{$R '..\res\version.res' '..\res\version.rc'}

const
  // DLL version
  VersionMajor = 1;
  VersionMinor = 10;
  VersionBuild = 5502;

  // list of compatible header versions
  CompatBuildList: array[0..3] of record
                                    Major,
                                    Minor,
                                    Build: Longword;
                                  end
                 = ((Major: 1; Minor: 10; Build: 5502),
                    (Major: 1; Minor: 10; Build: 4429),
                    (Major: 1; Minor: 10; Build: 0804),
                    (Major: 1; Minor: 00; Build: 0728));

function GetIMAManager(BlockSize, SampPerBlock, NumChannels: Longint;
  const FormatType: TGUID): Longint; stdcall;
begin
  Result := GetNewHandle(BlockSize, SampPerBlock, NumChannels, FormatType)
end;

procedure ReleaseIMAManager(Handle: Longint); stdcall;
begin
  ReleaseHandle(Handle)
end;

procedure IMALoadSamples(Handle: Longint; const InBuffer; NumSamples: Longint);
  stdcall;
var
  Handler   : THandler;
  SampBuffer: TIMA_ADPCM_SampleBuffer;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  SetLength(SampBuffer, NumSamples);
  Move(InBuffer, SampBuffer[0], Length(SampBuffer) * 2);

  Handler.Manager.AssignSampleBuffer(SampBuffer)
end;

procedure IMAUnloadSamples(Handle: Longint); stdcall;
var
  Handler: THandler;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Handler.Manager.UnassignSampleBuffer
end;

procedure IMALoadCodes(Handle: Longint; const InBuffer; CodeSize: Longint);
  stdcall;
var
  Handler   : THandler;
  CodeBuffer: TIMA_ADPCM_CodeBuffer;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  SetLength(CodeBuffer, CodeSize);
  Move(InBuffer, CodeBuffer[0], Length(CodeBuffer));

  Handler.Manager.AssignCodeBuffer(CodeBuffer)
end;

procedure IMAUnloadCodes(Handle: Longint); stdcall;
var
  Handler: THandler;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Handler.Manager.UnassignCodeBuffer
end;

function IMAGetCompBlockSize(Handle: Longint): Longint; stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetCompressedBlockSize
end;

function IMAGetDecompBlockSize(Handle: Longint): Longint; stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetDecompressedBlockSize
end;

function IMAGetCompressedSize(Handle: Longint): Longint; stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetCompressedSize
end;

function IMAGetDecompressedSize(Handle: Longint): Longint; stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetDecompressedSize
end;

function IMAGetCompSizeBySize(Handle: Longint; NumSamples: Longint): Longint;
  stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetCompressedSize(NumSamples)
end;

function IMAGetDecompSizeBySize(Handle: Longint; CodeSize: Longint): Longint;
  stdcall;
var
  Handler: THandler;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.GetDecompressedSize(CodeSize)
end;

function IMACompressBlock(Handle: Longint; var OutBuffer): Longint; stdcall;
var
  Handler   : THandler;
  CodeBuffer: TIMA_ADPCM_CodeBuffer;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.CompressBlock;

  Handler.Manager.GetCodeBuffer(CodeBuffer);
  Move(CodeBuffer[0], OutBuffer, Length(CodeBuffer))
end;

function IMADecompressBlock(Handle: Longint; var OutBuffer): Longint; stdcall;
var
  Handler   : THandler;
  SampBuffer: TIMA_ADPCM_SampleBuffer;
begin
  Result := 0;

  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Result := Handler.Manager.DecompressBlock;

  Handler.Manager.GetSampleBuffer(SampBuffer);
  Move(SampBuffer[0], OutBuffer, Length(SampBuffer) * 2)
end;

procedure IMACompress(Handle: Longint; var OutBuffer); stdcall;
var
  Handler   : THandler;
  CodeBuffer: TIMA_ADPCM_CodeBuffer;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Handler.Manager.Compress;

  Handler.Manager.GetCodeBuffer(CodeBuffer);
  Move(CodeBuffer[0], OutBuffer, Length(CodeBuffer))
end;

procedure IMADecompress(Handle: Longint; var OutBuffer); stdcall;
var
  Handler   : THandler;
  SampBuffer: TIMA_ADPCM_SampleBuffer;
begin
  if GetHandler(Handle, Handler) < Handler_Success then
    Exit;

  Handler.Manager.Decompress;

  Handler.Manager.GetSampleBuffer(SampBuffer);
  Move(SampBuffer[0], OutBuffer, Length(SampBuffer) * 2)
end;

procedure GetDLLVersion(var Major, Minor, Build: Longword);
          stdcall;
begin
  Major := VersionMajor;
  Minor := VersionMinor;
  Build := VersionBuild
end;

function IsHeaderSupported(ExtMajor, ExtMinor, ExtBuild: Longword): Boolean;
         stdcall;
var
  i: Longint;
begin
  Result := False;

  for i := 0 to High(CompatBuildList) do
    with CompatBuildList[i] do
      Result := Result or
        ((ExtMajor = Major) and (ExtMinor = Minor) and (ExtBuild = Build))
end;

exports
  GetIMAManager,
  ReleaseIMAManager,
  IMALoadSamples,
  IMAUnloadSamples,
  IMALoadCodes,
  IMAUnloadCodes,
  IMAGetCompBlockSize,
  IMAGetDecompBlockSize,
  IMAGetCompressedSize,
  IMAGetDecompressedSize,
  IMAGetCompSizeBySize,
  IMAGetDecompSizeBySize,
  IMACompressBlock,
  IMADecompressBlock,
  IMACompress,
  IMADecompress,
  GetDLLVersion,
  IsHeaderSupported;

end.
